@@ -27,15 +27,8 @@
 
 #define FILE_ATTRIBUTES	  \
 	G_FILE_ATTRIBUTE_STANDARD_NAME "," \
-	G_FILE_ATTRIBUTE_STANDARD_TYPE
-
-#define FILES_QUEUE_PROCESS_INTERVAL 2000
-#define FILES_QUEUE_PROCESS_MAX      5000
-
-/* This is the number of files to be called back with from GIO at a
- * time so we don't get called back for every file.
- */
-#define FILES_GROUP_SIZE             100
+	G_FILE_ATTRIBUTE_STANDARD_TYPE "," \
+	G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN
 
 #define MAX_SIMULTANEOUS_ITEMS       64
 
@@ -66,6 +59,8 @@ struct DirectoryProcessingData {
 };
 
 struct DirectoryRootInfo {
+	TrackerCrawler *crawler;
+	GTask *task;
 	GFile *directory;
 	GNode *tree;
 
@@ -75,6 +70,8 @@ struct DirectoryRootInfo {
 
 	DataProviderData *dpd;
 
+	guint idle_id;
+
 	/* Directory stats */
 	guint directories_found;
 	guint directories_ignored;
@@ -85,35 +82,15 @@ struct DirectoryRootInfo {
 struct TrackerCrawlerPrivate {
 	TrackerDataProvider *data_provider;
 
-	/* Directories to crawl */
-	GQueue         *directories;
-
-	GCancellable   *cancellable;
-
 	/* Idle handler for processing found data */
 	guint           idle_id;
 
-	gdouble         throttle;
-
 	gchar          *file_attributes;
 
-	/* Statistics */
-	GTimer         *timer;
-
-	/* Status */
-	gboolean        is_running;
-	gboolean        is_finished;
-	gboolean        is_paused;
-	gboolean        was_started;
-};
-
-enum {
-	CHECK_DIRECTORY,
-	CHECK_FILE,
-	CHECK_DIRECTORY_CONTENTS,
-	DIRECTORY_CRAWLED,
-	FINISHED,
-	LAST_SIGNAL
+	/* Check func */
+	TrackerCrawlerCheckFunc check_func;
+	gpointer check_func_data;
+	GDestroyNotify check_func_destroy;
 };
 
 enum {
@@ -130,11 +107,6 @@ static void     crawler_set_property     (GObject         *object,
                                           const GValue    *value,
                                           GParamSpec      *pspec);
 static void     crawler_finalize         (GObject         *object);
-static gboolean check_defaults           (TrackerCrawler  *crawler,
-                                          GFile           *file);
-static gboolean check_contents_defaults  (TrackerCrawler  *crawler,
-                                          GFile           *file,
-                                          GList           *contents);
 static void     data_provider_data_free  (DataProviderData        *dpd);
 
 static void     data_provider_begin      (TrackerCrawler          *crawler,
@@ -144,8 +116,6 @@ static void     data_provider_end        (TrackerCrawler          *crawler,
                                           DirectoryRootInfo       *info);
 static void     directory_root_info_free (DirectoryRootInfo *info);
 
-
-static guint signals[LAST_SIGNAL] = { 0, };
 static GQuark file_info_quark = 0;
 
 G_DEFINE_TYPE_WITH_PRIVATE (TrackerCrawler, tracker_crawler, G_TYPE_OBJECT)
@@ -154,73 +124,11 @@ static void
 tracker_crawler_class_init (TrackerCrawlerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	TrackerCrawlerClass *crawler_class = TRACKER_CRAWLER_CLASS (klass);
 
 	object_class->set_property = crawler_set_property;
 	object_class->get_property = crawler_get_property;
 	object_class->finalize = crawler_finalize;
 
-	crawler_class->check_directory = check_defaults;
-	crawler_class->check_file      = check_defaults;
-	crawler_class->check_directory_contents = check_contents_defaults;
-
-	signals[CHECK_DIRECTORY] =
-		g_signal_new ("check-directory",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (TrackerCrawlerClass, check_directory),
-		              tracker_accumulator_check_file,
-		              NULL,
-		              NULL,
-		              G_TYPE_BOOLEAN,
-		              1,
-		              G_TYPE_FILE);
-	signals[CHECK_FILE] =
-		g_signal_new ("check-file",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (TrackerCrawlerClass, check_file),
-		              tracker_accumulator_check_file,
-		              NULL,
-		              NULL,
-		              G_TYPE_BOOLEAN,
-		              1,
-		              G_TYPE_FILE);
-	signals[CHECK_DIRECTORY_CONTENTS] =
-		g_signal_new ("check-directory-contents",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (TrackerCrawlerClass, check_directory_contents),
-		              tracker_accumulator_check_file,
-		              NULL,
-		              NULL,
-		              G_TYPE_BOOLEAN,
-		              2, G_TYPE_FILE, G_TYPE_POINTER);
-	signals[DIRECTORY_CRAWLED] =
-		g_signal_new ("directory-crawled",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (TrackerCrawlerClass, directory_crawled),
-		              NULL, NULL,
-		              NULL,
-		              G_TYPE_NONE,
-		              6,
-			      G_TYPE_FILE,
-		              G_TYPE_POINTER,
-		              G_TYPE_UINT,
-		              G_TYPE_UINT,
-		              G_TYPE_UINT,
-		              G_TYPE_UINT);
-	signals[FINISHED] =
-		g_signal_new ("finished",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (TrackerCrawlerClass, finished),
-		              NULL, NULL,
-			      NULL,
-		              G_TYPE_NONE,
-		              1, G_TYPE_BOOLEAN);
-
 	g_object_class_install_property (object_class,
 	                                 PROP_DATA_PROVIDER,
 	                                 g_param_spec_object ("data-provider",
@@ -237,10 +145,6 @@ tracker_crawler_class_init (TrackerCrawlerClass *klass)
 static void
 tracker_crawler_init (TrackerCrawler *object)
 {
-	TrackerCrawlerPrivate *priv;
-
-	priv = tracker_crawler_get_instance_private (TRACKER_CRAWLER (object));
-	priv->directories = g_queue_new ();
 }
 
 static void
@@ -290,22 +194,10 @@ crawler_finalize (GObject *object)
 
 	priv = tracker_crawler_get_instance_private (TRACKER_CRAWLER (object));
 
-	if (priv->timer) {
-		g_timer_destroy (priv->timer);
+	if (priv->check_func_data && priv->check_func_destroy) {
+		priv->check_func_destroy (priv->check_func_data);
 	}
 
-	if (priv->idle_id) {
-		g_source_remove (priv->idle_id);
-	}
-
-	if (priv->cancellable) {
-		g_cancellable_cancel (priv->cancellable);
-		g_object_unref (priv->cancellable);
-	}
-
-	g_queue_foreach (priv->directories, (GFunc) directory_root_info_free, NULL);
-	g_queue_free (priv->directories);
-
 	g_free (priv->file_attributes);
 
 	if (priv->data_provider) {
@@ -315,21 +207,6 @@ crawler_finalize (GObject *object)
 	G_OBJECT_CLASS (tracker_crawler_parent_class)->finalize (object);
 }
 
-static gboolean
-check_defaults (TrackerCrawler *crawler,
-                GFile          *file)
-{
-	return TRUE;
-}
-
-static gboolean
-check_contents_defaults (TrackerCrawler  *crawler,
-                         GFile           *file,
-                         GList           *contents)
-{
-	return TRUE;
-}
-
 TrackerCrawler *
 tracker_crawler_new (TrackerDataProvider *data_provider)
 {
@@ -358,23 +235,52 @@ tracker_crawler_new (TrackerDataProvider *data_provider)
 	return crawler;
 }
 
+void
+tracker_crawler_set_check_func (TrackerCrawler          *crawler,
+                                TrackerCrawlerCheckFunc  func,
+                                gpointer                 user_data,
+                                GDestroyNotify           destroy_notify)
+{
+	TrackerCrawlerPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
+
+	priv = tracker_crawler_get_instance_private (crawler);
+	priv->check_func = func;
+	priv->check_func_data = user_data;
+	priv->check_func_destroy = destroy_notify;
+}
+
+static gboolean
+invoke_check (TrackerCrawler           *crawler,
+              TrackerCrawlerCheckFlags  flags,
+              GFile                    *file,
+              GFileInfo                *file_info,
+              GList                    *children)
+{
+	TrackerCrawlerPrivate *priv;
+
+	priv = tracker_crawler_get_instance_private (crawler);
+
+	if (!priv->check_func)
+		return TRUE;
+
+	return priv->check_func (crawler, flags,
+	                         file, file_info, children,
+	                         priv->check_func_data);
+}
+
 static gboolean
 check_file (TrackerCrawler    *crawler,
 	    DirectoryRootInfo *info,
             GFile             *file)
 {
+	GFileInfo *file_info;
 	gboolean use = FALSE;
-	TrackerCrawlerPrivate *priv;
 
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	g_signal_emit (crawler, signals[CHECK_FILE], 0, file, &use);
-
-	/* Crawler may have been stopped while waiting for the 'use' value,
-	 * and the DirectoryRootInfo already disposed... */
-	if (!priv->is_running) {
-		return FALSE;
-	}
+	file_info = g_object_get_qdata (G_OBJECT (file), file_info_quark);
+	use = invoke_check (crawler, TRACKER_CRAWLER_CHECK_FILE,
+	                    file, file_info, NULL);
 
 	info->files_found++;
 
@@ -390,18 +296,12 @@ check_directory (TrackerCrawler    *crawler,
 		 DirectoryRootInfo *info,
 		 GFile             *file)
 {
+	GFileInfo *file_info;
 	gboolean use = FALSE;
-	TrackerCrawlerPrivate *priv;
 
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	g_signal_emit (crawler, signals[CHECK_DIRECTORY], 0, file, &use);
-
-	/* Crawler may have been stopped while waiting for the 'use' value,
-	 * and the DirectoryRootInfo already disposed... */
-	if (!priv->is_running) {
-		return FALSE;
-	}
+	file_info = g_object_get_qdata (G_OBJECT (file), file_info_quark);
+	use = invoke_check (crawler, TRACKER_CRAWLER_CHECK_DIRECTORY,
+	                    file, file_info, NULL);
 
 	info->directories_found++;
 
@@ -465,6 +365,7 @@ directory_processing_data_add_child (DirectoryProcessingData *data,
 
 static DirectoryRootInfo *
 directory_root_info_new (GFile                 *file,
+                         GFileInfo             *file_info,
                          gchar                 *file_attributes,
                          TrackerDirectoryFlags  flags)
 {
@@ -485,19 +386,7 @@ directory_root_info_new (GFile                 *file,
 		allow_stat = FALSE;
 	}
 
-	/* NOTE: GFileInfo is ABSOLUTELY required here, without it the
-	 * TrackerFileNotifier will think that top level roots have
-	 * been deleted because the GFileInfo GQuark does not exist.
-	 *
-	 * This is seen easily by mounting a removable device,
-	 * indexing, then removing, then re-inserting that same
-	 * device.
-	 *
-	 * The check is done later in the TrackerFileNotifier by
-	 * looking up the qdata that we set in both conditions below.
-	 */
-	if (allow_stat && file_attributes) {
-		GFileInfo *file_info;
+	if (!file_info && allow_stat && file_attributes) {
 		GFileQueryInfoFlags file_flags;
 
 		file_flags = G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS;
@@ -511,8 +400,7 @@ directory_root_info_new (GFile                 *file,
 		                         file_info_quark,
 		                         file_info,
 		                         (GDestroyNotify) g_object_unref);
-	} else {
-		GFileInfo *file_info;
+	} else if (!file_info) {
 		gchar *basename;
 
 		file_info = g_file_info_new ();
@@ -555,6 +443,10 @@ directory_tree_free_foreach (GNode    *node,
 static void
 directory_root_info_free (DirectoryRootInfo *info)
 {
+	if (info->idle_id) {
+		g_source_remove (info->idle_id);
+	}
+
 	if (info->dpd)  {
 		data_provider_end (info->dpd->crawler, info);
 	}
@@ -578,46 +470,22 @@ directory_root_info_free (DirectoryRootInfo *info)
 }
 
 static gboolean
-process_next (TrackerCrawler *crawler)
+process_next (DirectoryRootInfo *info)
 {
-	TrackerCrawlerPrivate   *priv;
-	DirectoryRootInfo       *info;
-	DirectoryProcessingData *dir_data = NULL;
-	gboolean                 stop_idle = FALSE;
+	DirectoryProcessingData *dir_data;
+	GTask *task = info->task;
 
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	if (priv->is_paused) {
-		/* Stop the idle func for now until we are unpaused */
-		priv->idle_id = 0;
-
-		return FALSE;
+	if (g_task_return_error_if_cancelled (task)) {
+		g_object_unref (task);
+		return G_SOURCE_REMOVE;
 	}
 
-	info = g_queue_peek_head (priv->directories);
-
-	if (info) {
-		dir_data = g_queue_peek_head (info->directory_processing_queue);
-	}
+	dir_data = g_queue_peek_head (info->directory_processing_queue);
 
 	if (dir_data) {
 		/* One directory inside the tree hierarchy is being inspected */
-		if (!dir_data->was_inspected) {
-			dir_data->was_inspected = TRUE;
-
-			/* Crawler may have been already stopped while we were waiting for the
-			 *  check_directory return value, and thus we should check if it's
-			 *  running before going on with the iteration */
-			if (priv->is_running && G_NODE_IS_ROOT (dir_data->node)) {
-				/* Directory contents haven't been inspected yet,
-				 * stop this idle function while it's being iterated
-				 */
-				data_provider_begin (crawler, info, dir_data);
-				stop_idle = TRUE;
-			}
-		} else if (dir_data->was_inspected &&
-			   !dir_data->ignored_by_content &&
-			   dir_data->children != NULL) {
+		if (!dir_data->ignored_by_content &&
+		    dir_data->children != NULL) {
 			DirectoryChildData *child_data;
 			GNode *child_node = NULL;
 
@@ -629,18 +497,14 @@ process_next (TrackerCrawler *crawler)
 			dir_data->children = g_slist_remove (dir_data->children, child_data);
 
 			if (((child_data->is_dir &&
-			      check_directory (crawler, info, child_data->child)) ||
+			      check_directory (info->crawler, info, child_data->child)) ||
 			     (!child_data->is_dir &&
-			      check_file (crawler, info, child_data->child))) &&
-			    /* Crawler may have been already stopped while we were waiting for the
-			     *	check_directory or check_file return value, and thus we should
-			     *	 check if it's running before going on */
-			    priv->is_running) {
+			      check_file (info->crawler, info, child_data->child)))) {
 				child_node = g_node_prepend_data (dir_data->node,
 								  g_object_ref (child_data->child));
 			}
 
-			if (G_NODE_IS_ROOT (dir_data->node) && priv->is_running &&
+			if (G_NODE_IS_ROOT (dir_data->node) &&
 			    child_node && child_data->is_dir) {
 				DirectoryProcessingData *child_dir_data;
 
@@ -649,52 +513,38 @@ process_next (TrackerCrawler *crawler)
 			}
 
 			directory_child_data_free (child_data);
+
+			return G_SOURCE_CONTINUE;
 		} else {
 			/* No (more) children, or directory ignored. stop processing. */
 			g_queue_pop_head (info->directory_processing_queue);
+			g_task_return_boolean (task, !dir_data->ignored_by_content);
 			directory_processing_data_free (dir_data);
+			g_object_unref (task);
 		}
-	} else if (!dir_data && info) {
+	} else {
 		/* Current directory being crawled doesn't have anything else
-		 * to process, emit ::directory-crawled and free data.
+		 * to process.
 		 */
-		g_signal_emit (crawler, signals[DIRECTORY_CRAWLED], 0,
-			       info->directory,
-			       info->tree,
-			       info->directories_found,
-			       info->directories_ignored,
-			       info->files_found,
-			       info->files_ignored);
+		g_task_return_boolean (task, TRUE);
+		g_object_unref (task);
 
-		data_provider_end (crawler, info);
-		g_queue_pop_head (priv->directories);
-		directory_root_info_free (info);
+		data_provider_end (info->crawler, info);
 	}
 
-	if (!g_queue_peek_head (priv->directories)) {
-		/* There's nothing else to process */
-		priv->is_finished = TRUE;
-		tracker_crawler_stop (crawler);
-		stop_idle = TRUE;
-	}
-
-	if (stop_idle) {
-		priv->idle_id = 0;
-		return FALSE;
-	}
-
-	return TRUE;
+	/* There's nothing else to process */
+	return G_SOURCE_REMOVE;
 }
 
 static gboolean
 process_func (gpointer data)
 {
-	TrackerCrawler *crawler = data;
+	DirectoryRootInfo *info = data;
 	gboolean retval = FALSE;
 	gint i;
 
 	for (i = 0; i < MAX_SIMULTANEOUS_ITEMS; i++) {
-		retval = process_next (crawler);
+		retval = process_next (info);
 		if (retval == FALSE)
 			break;
 	}
@@ -703,40 +553,15 @@ process_func (gpointer data)
 }
 
 static gboolean
-process_func_start (TrackerCrawler *crawler)
+process_func_start (DirectoryRootInfo *info)
 {
-	TrackerCrawlerPrivate *priv;
-
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	if (priv->is_paused) {
-		return FALSE;
-	}
-
-	if (priv->is_finished) {
-		return FALSE;
-	}
-
-	if (priv->idle_id == 0) {
-		priv->idle_id = g_idle_add (process_func, crawler);
+	if (info->idle_id == 0) {
+		info->idle_id = g_idle_add (process_func, info);
 	}
 
 	return TRUE;
 }
 
-static void
-process_func_stop (TrackerCrawler *crawler)
-{
-	TrackerCrawlerPrivate *priv;
-
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	if (priv->idle_id != 0) {
-		g_source_remove (priv->idle_id);
-		priv->idle_id = 0;
-	}
-}
-
 static DataProviderData *
 data_provider_data_new (TrackerCrawler          *crawler,
                         DirectoryRootInfo       *root_info,
@@ -762,6 +587,7 @@ data_provider_data_process (DataProviderData *dpd)
 	TrackerCrawler *crawler;
 	GSList *l;
 	GList *children = NULL;
+	GFileInfo *file_info;
 	gboolean use;
 
 	crawler = dpd->crawler;
@@ -773,7 +599,9 @@ data_provider_data_process (DataProviderData *dpd)
 		children = g_list_prepend (children, child_data->child);
 	}
 
-	g_signal_emit (crawler, signals[CHECK_DIRECTORY_CONTENTS], 0, dpd->dir_file, children, &use);
+	file_info = g_object_get_qdata (G_OBJECT (dpd->dir_file), file_info_quark);
+	use = invoke_check (crawler, TRACKER_CRAWLER_CHECK_CONTENT,
+	                    dpd->dir_file, file_info, children);
 	g_list_free (children);
 
 	if (!use) {
@@ -904,10 +732,12 @@ enumerate_next_cb (GObject      *object,
 {
 	DataProviderData *dpd;
 	GList *info;
-	g_autoptr(GError) error = NULL;
+	GError *error = NULL;
+	DirectoryRootInfo *root_info;
 
 	info = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (object), result, &error);
 	dpd = user_data;
+	root_info = dpd->root_info;
 
 	if (!info) {
 		/* Could be due to:
@@ -918,32 +748,27 @@ enumerate_next_cb (GObject      *object,
 			/* We don't consider cancellation an error, so we only
 			 * log errors which are not cancellations.
 			 */
-			if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-				gchar *uri = g_file_get_uri (dpd->dir_file);
-				g_warning ("Could not enumerate next item in container / directory '%s', %s",
-				           uri, error ? error->message : "no error given");
-				g_free (uri);
-			} else {
-				return;
-			}
+			g_task_return_error (root_info->task, error);
+			g_object_unref (root_info->task);
+			return;
 		} else {
 			/* Done enumerating, start processing what we got ... */
 			data_provider_data_add (dpd);
 			data_provider_data_process (dpd);
 		}
 
-		process_func_start (dpd->crawler);
+		process_func_start (dpd->root_info);
 	} else {
-		TrackerCrawlerPrivate *priv;
+		DirectoryRootInfo *root_info;
 
-		priv = tracker_crawler_get_instance_private (dpd->crawler);
+		root_info = dpd->root_info;
 
 		/* More work to do, we keep reference given to us */
 		dpd->files = g_list_concat (dpd->files, info);
 		g_file_enumerator_next_files_async (G_FILE_ENUMERATOR (object),
 		                                    MAX_SIMULTANEOUS_ITEMS,
 		                                    G_PRIORITY_LOW,
-		                                    priv->cancellable,
+		                                    g_task_get_cancellable (root_info->task),
 		                                    enumerate_next_cb,
 		                                    dpd);
 	}
@@ -954,7 +779,6 @@ data_provider_begin_cb (GObject      *object,
                         GAsyncResult *result,
                         gpointer      user_data)
 {
-	TrackerCrawlerPrivate *priv;
 	GFileEnumerator *enumerator;
 	DirectoryRootInfo *info;
 	DataProviderData *dpd;
@@ -965,27 +789,17 @@ data_provider_begin_cb (GObject      *object,
 	info = user_data;
 
 	if (error) {
-		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-			gchar *uri;
-
-			dpd = info->dpd;
-			uri = g_file_get_uri (dpd->dir_file);
-			g_warning ("Could not enumerate container / directory '%s', %s",
-			           uri, error ? error->message : "no error given");
-			g_free (uri);
-			process_func_start (dpd->crawler);
-		}
-		g_clear_error (&error);
+		g_task_return_error (info->task, error);
+		g_object_unref (info->task);
 		return;
 	}
 
 	dpd = info->dpd;
 	dpd->enumerator = enumerator;
-	priv = tracker_crawler_get_instance_private (dpd->crawler);
 	g_file_enumerator_next_files_async (enumerator,
 	                                    MAX_SIMULTANEOUS_ITEMS,
 	                                    G_PRIORITY_LOW,
-	                                    priv->cancellable,
+	                                    g_task_get_cancellable (info->task),
 	                                    enumerate_next_cb,
 	                                    dpd);
 }
@@ -1001,12 +815,6 @@ data_provider_begin (TrackerCrawler          *crawler,
 
 	priv = tracker_crawler_get_instance_private (crawler);
 
-	/* DataProviderData is freed in data_provider_end() call. This
-	 * call must _ALWAYS_ be reached even on cancellation or
-	 * failure, this is normally the case when we return to the
-	 * process_func() and finish a directory.
-	 */
-	dir_data->was_inspected = TRUE;
 	dpd = data_provider_data_new (crawler, info, dir_data);
 	info->dpd = dpd;
 
@@ -1023,189 +831,86 @@ data_provider_begin (TrackerCrawler          *crawler,
 	                                   attrs,
 	                                   info->flags,
 	                                   G_PRIORITY_LOW,
-	                                   priv->cancellable,
+	                                   g_task_get_cancellable (info->task),
 	                                   data_provider_begin_cb,
 	                                   info);
 	g_free (attrs);
 }
 
 gboolean
-tracker_crawler_start (TrackerCrawler        *crawler,
-                       GFile                 *file,
-                       TrackerDirectoryFlags  flags)
+tracker_crawler_get_finish (TrackerCrawler  *crawler,
+                            GAsyncResult    *result,
+                            GFile          **directory,
+                            GNode          **tree,
+                            guint           *directories_found,
+                            guint           *directories_ignored,
+                            guint           *files_found,
+                            guint           *files_ignored,
+                            GError         **error)
+{
+	DirectoryRootInfo *info;
+	gboolean retval;
+
+	info = g_task_get_task_data (G_TASK (result));
+
+	retval = g_task_propagate_boolean (G_TASK (result), error);
+	if (retval) {
+		if (tree)
+			*tree = info->tree;
+	}
+
+	if (directory)
+		*directory = info->directory;
+	if (directories_found)
+		*directories_found = info->directories_found;
+	if (directories_ignored)
+		*directories_ignored = info->directories_ignored;
+	if (files_found)
+		*files_found = info->files_found;
+	if (files_ignored)
+		*files_ignored = info->files_ignored;
+
+	return retval;
+}
+
+void
+tracker_crawler_get (TrackerCrawler        *crawler,
+                     GFile                 *file,
+                     TrackerDirectoryFlags  flags,
+                     GCancellable          *cancellable,
+                     GAsyncReadyCallback    callback,
+                     gpointer               user_data)
 {
 	TrackerCrawlerPrivate *priv;
 	DirectoryProcessingData *dir_data;
 	DirectoryRootInfo *info;
-	gboolean enable_stat;
+	GFileInfo *file_info;
+	GTask *task;
 
-	g_return_val_if_fail (TRACKER_IS_CRAWLER (crawler), FALSE);
-	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
+	g_return_if_fail (G_IS_FILE (file));
 
 	priv = tracker_crawler_get_instance_private (crawler);
 
-	enable_stat = (flags & TRACKER_DIRECTORY_FLAG_NO_STAT) == 0;
+	file_info = tracker_crawler_get_file_info (crawler, file);
 
-	if (enable_stat && !g_file_query_exists (file, NULL)) {
-		/* This shouldn't happen, unless the removal/unmount notification
-		 * didn't yet reach the TrackerFileNotifier.
-		 */
-		return FALSE;
+	info = directory_root_info_new (file, file_info, priv->file_attributes, flags);
+	task = g_task_new (crawler, cancellable, callback, user_data);
+	g_task_set_task_data (task, info,
+	                      (GDestroyNotify) directory_root_info_free);
+	info->task = task;
+	info->crawler = crawler;
+
+	if (!file_info && !check_directory (crawler, info, file)) {
+		g_task_return_boolean (task, FALSE);
+		g_object_unref (task);
+		return;
 	}
 
-	priv->was_started = TRUE;
-
-	/* Time the event */
-	if (priv->timer) {
-		g_timer_destroy (priv->timer);
-	}
-
-	priv->timer = g_timer_new ();
-
-	if (priv->is_paused) {
-		g_timer_stop (priv->timer);
-	}
-
-	/* Set a brand new cancellable */
-	if (priv->cancellable) {
-		g_cancellable_cancel (priv->cancellable);
-		g_object_unref (priv->cancellable);
-	}
-
-	priv->cancellable = g_cancellable_new ();
-
-	/* Set as running now */
-	priv->is_running = TRUE;
-	priv->is_finished = FALSE;
-
-	info = directory_root_info_new (file, priv->file_attributes, flags);
-
-	if (!check_directory (crawler, info, file)) {
-		directory_root_info_free (info);
-
-		g_timer_destroy (priv->timer);
-		priv->timer = NULL;
-
-		priv->is_running = FALSE;
-		priv->is_finished = TRUE;
-
-		return FALSE;
-	}
-
-	g_queue_push_tail (priv->directories, info);
-
 	dir_data = g_queue_peek_head (info->directory_processing_queue);
 
 	if (dir_data)
 		data_provider_begin (crawler, info, dir_data);
-
-	return TRUE;
-}
-
-void
-tracker_crawler_stop (TrackerCrawler *crawler)
-{
-	TrackerCrawlerPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
-
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	/* If already not running, just ignore */
-	if (!priv->is_running) {
-		return;
-	}
-
-	priv->is_running = FALSE;
-	g_cancellable_cancel (priv->cancellable);
-
-	process_func_stop (crawler);
-
-	if (priv->timer) {
-		g_timer_destroy (priv->timer);
-		priv->timer = NULL;
-	}
-
-	/* Clean up queue */
-	g_queue_foreach (priv->directories, (GFunc) directory_root_info_free, NULL);
-	g_queue_clear (priv->directories);
-
-	g_signal_emit (crawler, signals[FINISHED], 0,
-	               !priv->is_finished);
-
-	/* We don't free the queue in case the crawler is reused, it
-	 * is only freed in finalize.
-	 */
-}
-
-void
-tracker_crawler_pause (TrackerCrawler *crawler)
-{
-	TrackerCrawlerPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
-
-	priv = tracker_crawler_get_instance_private (crawler);
-	priv->is_paused = TRUE;
-
-	if (priv->is_running) {
-		g_timer_stop (priv->timer);
-		process_func_stop (crawler);
-	}
-
-	g_message ("Crawler is paused, %s",
-	           priv->is_running ? "currently running" : "not running");
-}
-
-void
-tracker_crawler_resume (TrackerCrawler *crawler)
-{
-	TrackerCrawlerPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
-
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	priv->is_paused = FALSE;
-
-	if (priv->is_running) {
-		g_timer_continue (priv->timer);
-		process_func_start (crawler);
-	}
-
-	g_message ("Crawler is resuming, %s",
-	           priv->is_running ? "currently running" : "not running");
-}
-
-void
-tracker_crawler_set_throttle (TrackerCrawler *crawler,
-                              gdouble         throttle)
-{
-	TrackerCrawlerPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
-
-	priv = tracker_crawler_get_instance_private (crawler);
-
-	throttle = CLAMP (throttle, 0, 1);
-	priv->throttle = throttle;
-
-	/* Update timeouts */
-	if (priv->idle_id != 0) {
-		guint interval, idle_id;
-
-		interval = TRACKER_CRAWLER_MAX_TIMEOUT_INTERVAL * priv->throttle;
-
-		g_source_remove (priv->idle_id);
-
-		if (interval == 0) {
-			idle_id = g_idle_add (process_func, crawler);
-		} else {
-			idle_id = g_timeout_add (interval, process_func, crawler);
-		}
-
-		priv->idle_id = idle_id;
-	}
 }
 
 /**
@@ -1270,6 +975,6 @@ tracker_crawler_get_file_info (TrackerCrawler *crawler,
 	g_return_val_if_fail (TRACKER_IS_CRAWLER (crawler), NULL);
 	g_return_val_if_fail (G_IS_FILE (file), NULL);
 
-	info = g_object_steal_qdata (G_OBJECT (file), file_info_quark);
+	info = g_object_get_qdata (G_OBJECT (file), file_info_quark);
 	return info;
 }
