@@ -1388,11 +1388,11 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
 
 	indexing_tree = tracker_miner_fs_get_indexing_tree (TRACKER_MINER_FS (mf));
 	tracker_indexing_tree_remove (indexing_tree, mount_root);
-	g_object_unref (mount_root);
 
 	/* Set mount point status in tracker-store */
 	set_up_mount_point (mf, mount_root, FALSE, NULL);
 
+	g_object_unref (mount_root);
 	g_free (uri);
 }
 
@@ -2062,6 +2062,7 @@ miner_files_process_file (TrackerMinerFS      *fs,
                           gboolean             create)
 {
 	TrackerMinerFilesPrivate *priv;
+	TrackerIndexingTree *indexing_tree;
 	TrackerResource *resource = NULL, *folder_resource = NULL, *graph_file = NULL;
 	const gchar *mime_type, *graph;
 	gchar *parent_urn;
@@ -2081,6 +2082,7 @@ miner_files_process_file (TrackerMinerFS      *fs,
 
 	priv->start_extractor = TRUE;
 	uri = g_file_get_uri (file);
+	indexing_tree = tracker_miner_fs_get_indexing_tree (fs);
 	mime_type = g_file_info_get_content_type (file_info);
 
 	is_directory = (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY ?
@@ -2156,7 +2158,7 @@ miner_files_process_file (TrackerMinerFS      *fs,
 	/* The URL of the DataObject (because IE = DO, this is correct) */
 	tracker_resource_set_string (resource, "nie:url", uri);
 
-	if (is_directory) {
+	if (is_directory || tracker_indexing_tree_file_is_root (indexing_tree, file)) {
 		folder_resource =
 			miner_files_create_folder_information_element (TRACKER_MINER_FILES (fs),
 								       file,
@@ -2168,9 +2170,10 @@ miner_files_process_file (TrackerMinerFS      *fs,
 
 	graph = tracker_extract_module_manager_get_graph (mime_type);
 
-	if (graph) {
+	if (graph && g_file_info_get_size (file_info) > 0) {
 		/* This mimetype will be extracted by some module, pre-fill the
 		 * nfo:FileDataObject in that graph.
+		 * Empty files skipped as mime-type for those cannot be trusted.
 		 */
 		graph_file = tracker_resource_new (uri);
 		tracker_resource_add_uri (graph_file, "rdf:type", "nfo:FileDataObject");
@@ -2179,6 +2182,9 @@ miner_files_process_file (TrackerMinerFS      *fs,
 		                             g_file_info_get_display_name (file_info));
 
 		tracker_resource_set_datetime (graph_file, "nfo:fileLastModified", modified);
+
+		tracker_resource_set_int64 (graph_file, "nfo:fileSize",
+		                            g_file_info_get_size (file_info));
 	}
 
 	if (delete_properties_sparql)
@@ -2204,8 +2210,9 @@ miner_files_process_file_attributes (TrackerMinerFS      *fs,
                                      GFileInfo           *info,
                                      TrackerSparqlBuffer *buffer)
 {
-	TrackerResource *resource;
+	TrackerResource *resource, *graph_file;
 	gchar *uri;
+	const gchar *mime_type, *graph;
 	GDateTime *modified;
 #ifdef GIO_SUPPORTS_CREATION_TIME
 	GDateTime *accessed, *created;
@@ -2230,8 +2237,17 @@ miner_files_process_file_attributes (TrackerMinerFS      *fs,
 	if (!modified)
 		modified = g_date_time_new_from_unix_utc (0);
 
+	mime_type = g_file_info_get_content_type (info);
+	graph = tracker_extract_module_manager_get_graph (mime_type);
+
 	/* Update nfo:fileLastModified */
 	tracker_resource_set_datetime (resource, "nfo:fileLastModified", modified);
+	if (graph) {
+		graph_file = tracker_resource_new (uri);
+		tracker_resource_set_datetime (graph_file, "nfo:fileLastModified", modified);
+		tracker_sparql_buffer_push (buffer, file, graph, graph_file);
+		g_clear_object (&graph_file);
+	}
 	g_date_time_unref (modified);
 
 #ifdef GIO_SUPPORTS_CREATION_TIME
@@ -2286,8 +2302,6 @@ add_delete_sparql (GFile               *file,
 
 	uri = g_file_get_uri (file);
 
-	sparql = g_string_new (NULL);
-
 	if (delete_children) {
 		sparql = g_string_new ("DELETE { "
 				       "  GRAPH " DEFAULT_GRAPH " {"
@@ -2311,6 +2325,8 @@ add_delete_sparql (GFile               *file,
 		g_string_append_printf (sparql, "STRSTARTS (?u, \"%s/\")", uri);
 
 		g_string_append (sparql, ")}");
+	} else {
+		sparql = g_string_new (NULL);
 	}
 
 	if (delete_self) {
@@ -2455,14 +2471,16 @@ miner_files_move_file (TrackerMinerFS      *fs,
 	                        "} INSERT {"
 	                        "  GRAPH ?g {"
 	                        "    <%s> a nfo:FileDataObject ; "
+	                        "         nfo:fileName \"%s\" ; "
 	                        "         ?p ?o "
 	                        "  }"
 	                        "} WHERE {"
 	                        "  GRAPH ?g {"
 	                        "    <%s> ?p ?o "
 	                        "  }"
+	                        "  FILTER (?p != nfo:fileName) . "
 	                        "}",
-	                        source_uri, uri, source_uri);
+	                        source_uri, uri, display_name, source_uri);
 	g_free (container_clause);
 
 	if (recursive) {
