@@ -53,24 +53,6 @@ TrackerMiner *tracker_miner_files_new                      (TrackerSparqlConnect
                                                             const gchar              *domain,
                                                             GError                  **error);
 
-/* Convenience functions for --eligible tracker-miner-fs cmdline */
-gboolean      tracker_miner_files_check_file               (GFile             *file,
-                                                            GSList            *ignored_file_paths,
-                                                            GSList            *ignored_file_patterns);
-gboolean      tracker_miner_files_check_directory          (GFile             *file,
-                                                            GSList            *index_recursive_directories,
-                                                            GSList            *index_single_directories,
-                                                            GSList            *ignored_directory_paths,
-                                                            GSList            *ignored_directory_patterns);
-gboolean      tracker_miner_files_check_directory_contents (GFile             *parent,
-                                                            GList             *children,
-                                                            GSList            *ignored_content);
-gboolean      tracker_miner_files_monitor_directory        (GFile             *file,
-                                                            gboolean           enable_monitors,
-                                                            GSList            *directories_to_check);
-gboolean      tracker_miner_files_is_file_eligible         (TrackerMinerFiles *miner,
-                                                            GFile             *file);
-
 /* Global functions to handle timestamp files */
 gboolean tracker_miner_files_get_first_index_done (TrackerMinerFiles *mf);
 void     tracker_miner_files_set_first_index_done (TrackerMinerFiles *mf,
