@@ -139,14 +139,13 @@ raw_profile_new (const gchar *input,
 		size++;
 	} while (g_ascii_isdigit (*ptr));
 
-	length_str = g_strndup (length_ptr, size - 1);
-
 	if (*ptr != '\n') {
 		return NULL;
 	}
 
 	ptr++;
 
+	length_str = g_strndup (length_ptr, size - 1);
 	length = atoi (length_str);
 	g_free (length_str);
 
@@ -230,7 +229,7 @@ read_metadata (TrackerResource      *metadata,
 				continue;
 			}
 
-			if (g_strcmp0 ("Raw profile type xmp", text_ptr[i].key) == 0) {
+			if (!xd && g_strcmp0 ("Raw profile type xmp", text_ptr[i].key) == 0) {
 				gchar *xmp_buffer;
 				guint xmp_buffer_length = 0;
 				guint input_len;
@@ -258,7 +257,7 @@ read_metadata (TrackerResource      *metadata,
 #endif /*HAVE_EXEMPI && PNG_iTXt_SUPPORTED */
 
 #if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
-			if (g_strcmp0 ("Raw profile type exif", text_ptr[i].key) == 0) {
+			if (!ed && g_strcmp0 ("Raw profile type exif", text_ptr[i].key) == 0) {
 				gchar *exif_buffer;
 				guint exif_buffer_length = 0;
 				guint input_len;
