@@ -5,7 +5,6 @@
 typedef struct {
 	TrackerMinerFS parent_instance;
 	guint n_process_file;
-	guint n_events;
 	guint finished : 1;
 } TestMiner;
 
@@ -28,32 +27,25 @@ G_DEFINE_TYPE (TestMiner, test_miner, TRACKER_TYPE_MINER_FS)
 	            TrackerMinerFSTestFixture, NULL, \
 	            fixture_setup, func, fixture_teardown)
 
-static gboolean
-test_miner_process_file (TrackerMinerFS *miner,
-                         GFile          *file,
-                         GTask          *task)
+static void
+test_miner_process_file (TrackerMinerFS      *miner,
+                         GFile               *file,
+                         GFileInfo           *info,
+                         TrackerSparqlBuffer *buffer,
+                         gboolean             created)
 {
 	TrackerResource *resource;
-	GError *error = NULL;
-	GFileInfo *info;
 	GDateTime *modification_time;
-	gchar *sparql, *str;
-	const gchar *urn;
+	TrackerIndexingTree *tree;
+	gchar *uri, *parent_uri;
 	GFile *parent;
 
 	((TestMiner *) miner)->n_process_file++;
-	info = g_file_query_info (file, "standard::*,time::*", 0, NULL, &error);
-	g_assert_no_error (error);
 
-	urn = tracker_miner_fs_get_folder_urn (miner, file);
-	if (g_strcmp0 (tracker_miner_fs_get_folder_urn (miner, file), urn) != 0) {
-		g_critical ("File %s did not get up to date URN",
-		            g_file_get_uri (file));
-	}
+	uri = g_file_get_uri (file);
+	resource = tracker_resource_new (uri);
 
-	resource = tracker_resource_new (tracker_miner_fs_get_folder_urn (miner, file));
-
-	if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY) {
+	if (info && g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY) {
 		tracker_resource_add_uri (resource, "rdf:type", "nfo:Folder");
 	}
 
@@ -62,43 +54,70 @@ test_miner_process_file (TrackerMinerFS *miner,
 	tracker_resource_add_relation (resource, "nie:interpretedAs", resource);
 	tracker_resource_add_relation (resource, "nie:isStoredAs", resource);
 
-	modification_time = g_file_info_get_modification_date_time (info);
-	if (modification_time) {
-		str = g_date_time_format_iso8601 (modification_time);
-		tracker_resource_set_string (resource, "nfo:fileLastModified", str);
-		g_free (str);
-		g_date_time_unref (modification_time);
+	if (info) {
+		modification_time = g_file_info_get_modification_date_time (info);
+		if (modification_time) {
+			tracker_resource_set_datetime (resource, "nfo:fileLastModified", modification_time);
+			g_date_time_unref (modification_time);
+		}
+#ifdef GIO_SUPPORTS_CREATION_TIME
+		creation_time = g_file_info_get_creation_date_time (info);
+		if (creation_time) {
+			tracker_resource_set_datetime (resource, "nfo:fileCreated", creation_time);
+			g_date_time_unref (creation_time);
+		}
+#endif
 	}
 
-	str = g_file_get_uri (file);
-	tracker_resource_set_string (resource, "nie:url", str);
-	g_free (str);
+	tracker_resource_set_string (resource, "nie:url", uri);
 
-	parent = g_file_get_parent (file);
-	urn = tracker_miner_fs_get_folder_urn (miner, parent);
-	g_object_unref (parent);
+	tree = tracker_miner_fs_get_indexing_tree (miner);
 
-	if (urn) {
-		tracker_resource_set_string (resource, "nfo:belongsToContainer", urn);
+	if (tracker_indexing_tree_file_is_root (tree, file)) {
+		tracker_resource_set_uri (resource, "nie:rootElementOf", uri);
+		tracker_resource_add_uri (resource, "rdf:type", "nie:DataSource");
+	} else {
+		GFile *root;
+		gchar *root_uri;
+
+		root = tracker_indexing_tree_get_root (tree, file, NULL);
+		if (root) {
+			root_uri = g_file_get_uri (root);
+			tracker_resource_set_uri (resource, "nie:dataSource", root_uri);
+			g_free (root_uri);
+		}
+
+		parent = g_file_get_parent (file);
+		parent_uri = g_file_get_uri (file);
+		tracker_resource_set_uri (resource, "nfo:belongsToContainer", parent_uri);
+		g_free (parent_uri);
+		g_object_unref (parent);
 	}
 
-	sparql = tracker_resource_print_sparql_update (resource, NULL, "Graph");
-	tracker_miner_fs_notify_finish (miner, task, sparql, NULL);
+	tracker_sparql_buffer_push (buffer, file, "tracker:FileSystem", resource);
 	g_object_unref (resource);
-	g_free (sparql);
-	g_object_unref (info);
-
-	return TRUE;
+	g_free (uri);
 }
 
-static gchar *
-test_miner_remove_file (TrackerMinerFS *miner,
-                        GFile          *file)
+static void
+test_miner_process_file_attributes (TrackerMinerFS      *miner,
+                                    GFile               *file,
+                                    GFileInfo           *info,
+                                    TrackerSparqlBuffer *buffer)
+{
+	test_miner_process_file (miner, file, info, buffer, FALSE);
+}
+
+static void
+test_miner_remove_file (TrackerMinerFS      *miner,
+                        GFile               *file,
+                        TrackerSparqlBuffer *buffer,
+                        gboolean             is_dir)
 {
 	gchar *sparql, *uri;
 
 	uri = g_file_get_uri (file);
-	sparql = g_strdup_printf ("WITH <Graph> "
+	sparql = g_strdup_printf ("WITH tracker:FileSystem "
 	                          "DELETE {"
 	                          "  ?u a rdfs:Resource . "
 	                          "} WHERE {"
@@ -107,17 +126,19 @@ test_miner_remove_file (TrackerMinerFS *miner,
 	                          "}", uri, uri);
 	g_free (uri);
 
-	return sparql;
+	tracker_sparql_buffer_push_sparql (buffer, file, sparql);
+	g_free (sparql);
 }
 
-static gchar *
-test_miner_remove_children (TrackerMinerFS *miner,
-                            GFile          *file)
+static void
+test_miner_remove_children (TrackerMinerFS      *miner,
+                            GFile               *file,
+                            TrackerSparqlBuffer *buffer)
 {
 	gchar *sparql, *uri;
 
 	uri = g_file_get_uri (file);
-	sparql = g_strdup_printf ("WITH <Graph> "
+	sparql = g_strdup_printf ("WITH tracker:FileSystem "
 	                          "DELETE {"
 	                          "  ?u a rdfs:Resource . "
 	                          "} WHERE {"
@@ -126,32 +147,20 @@ test_miner_remove_children (TrackerMinerFS *miner,
 	                          "}", uri);
 	g_free (uri);
 
-	return sparql;
+	tracker_sparql_buffer_push_sparql (buffer, file, sparql);
+	g_free (sparql);
 }
 
-static gchar *
-test_miner_move_file (TrackerMinerFS *miner,
-                      GFile          *dest,
-                      GFile          *source,
-                      gboolean        recursive)
+static void
+test_miner_move_file (TrackerMinerFS      *miner,
+                      GFile               *dest,
+                      GFile               *source,
+                      TrackerSparqlBuffer *buffer,
+                      gboolean             recursive)
 {
-	gchar *sparql, *uri, *dest_uri;
-
-	uri = g_file_get_uri (source);
-	dest_uri = g_file_get_uri (dest);
-	sparql = g_strdup_printf ("WITH <Graph> "
-	                          "DELETE {"
-	                          "  ?u nie:url ?url . "
-	                          "} INSERT {"
-	                          "  ?u nie:url '%s' . "
-	                          "} WHERE {"
-	                          "  ?u nie:url ?url . "
-	                          "  FILTER (?url = '%s')"
-	                          "}", dest_uri, uri);
-	g_free (dest_uri);
-	g_free (uri);
-
-	return sparql;
+	/* Caution: This does not deal with recursive moves */
+	test_miner_remove_file (miner, source, buffer, recursive);
+	test_miner_process_file (miner, dest, NULL, buffer, TRUE);
 }
 
 static void
@@ -171,7 +180,7 @@ test_miner_class_init (TestMinerClass *klass)
 	TrackerMinerFSClass *fs_class = TRACKER_MINER_FS_CLASS (klass);
 
 	fs_class->process_file = test_miner_process_file;
-	fs_class->process_file_attributes = test_miner_process_file;
+	fs_class->process_file_attributes = test_miner_process_file_attributes;
 	fs_class->remove_file = test_miner_remove_file;
 	fs_class->remove_children = test_miner_remove_children;
 	fs_class->move_file = test_miner_move_file;
@@ -191,6 +200,7 @@ test_miner_new (TrackerSparqlConnection  *conn,
 	return g_initable_new (test_miner_get_type (),
 	                       NULL, error,
 	                       "connection", conn,
+	                       "file-attributes", "standard::*,time::*",
 	                       NULL);
 }
 
@@ -209,7 +219,6 @@ static void
 test_miner_reset_counters (TestMiner *miner)
 {
 	miner->n_process_file = 0;
-	miner->n_events = 0;
 }
 
 static void
@@ -252,18 +261,16 @@ fixture_setup (TrackerMinerFSTestFixture *fixture,
                gconstpointer              data)
 {
 	GError *error = NULL;
-	GFile *file, *ontology;
+	GFile *ontology;
 	gchar *path;
 
 	path = g_build_filename (g_get_tmp_dir (), "tracker-miner-fs-test-XXXXXX", NULL);
 	fixture->test_root_path = g_mkdtemp_full (path, 0700);
 	fixture->test_root = g_file_new_for_path (fixture->test_root_path);
 
-	file = g_file_get_child (fixture->test_root, ".db");
 	ontology = g_file_new_for_path (TEST_ONTOLOGIES_DIR);
-	fixture->connection = tracker_sparql_connection_new (0, file, ontology, NULL, &error);
+	fixture->connection = tracker_sparql_connection_new (0, NULL, ontology, NULL, &error);
 	g_assert_no_error (error);
-	g_object_unref (file);
 	g_object_unref (ontology);
 
 	fixture->miner = test_miner_new (fixture->connection, &error);
@@ -393,16 +400,24 @@ fixture_iterate (TrackerMinerFSTestFixture *fixture)
 		g_main_context_iteration (NULL, TRUE);
 }
 
-static void
-fixture_iterate_filter (TrackerMinerFSTestFixture *fixture,
-                        guint                      n_events)
+static gboolean
+loop_timeout (GMainLoop *main_loop)
 {
-	TestMiner *miner = (TestMiner *) fixture->miner;
+	g_main_loop_quit (main_loop);
+	return G_SOURCE_REMOVE;
+}
 
-	miner->n_events = 0;
+static void
+fixture_iterate_timed (TrackerMinerFSTestFixture *fixture,
+                       int                        seconds)
+{
+	GMainLoop *main_loop;
 
-	while (miner->n_events < n_events)
-		g_main_context_iteration (NULL, TRUE);
+	main_loop = g_main_loop_new (NULL, FALSE);
+	g_timeout_add_seconds (seconds, (GSourceFunc) loop_timeout, main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
 }
 
 static void
@@ -997,8 +1012,6 @@ test_content_filter_on_parent_root (TrackerMinerFSTestFixture *fixture,
 			 "non-recursive/recursive/a/d,"
 			 "non-recursive/recursive/c");
 	g_free (content);
-
-	g_assert_cmpint (((TestMiner *) fixture->miner)->n_events, ==, 0);
 }
 
 static void
@@ -1135,6 +1148,7 @@ test_non_monitored_delete (TrackerMinerFSTestFixture *fixture,
 	DELETE_FOLDER (fixture, "recursive/1/2");
 
 	fixture_add_indexed_folder (fixture, "recursive",
+	                            TRACKER_DIRECTORY_FLAG_PRESERVE |
 	                            TRACKER_DIRECTORY_FLAG_CHECK_MTIME |
 	                            TRACKER_DIRECTORY_FLAG_CHECK_DELETED |
 	                            TRACKER_DIRECTORY_FLAG_RECURSE);
@@ -1406,7 +1420,6 @@ test_monitored_move (TrackerMinerFSTestFixture *fixture,
 	g_free (content);
 
 	MOVE_FILE (fixture, "recursive/a", "non-recursive/e");
-	MOVE_FILE (fixture, "recursive/1", "non-recursive/3");
 	MOVE_FILE (fixture, "recursive/d", "not-indexed/f");
 	MOVE_FILE (fixture, "non-recursive/2", "recursive/4");
 
@@ -1419,9 +1432,10 @@ test_monitored_move (TrackerMinerFSTestFixture *fixture,
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
 	                 "non-recursive,"
-	                 "non-recursive/3,"
 	                 "non-recursive/e,"
 	                 "recursive,"
+	                 "recursive/1,"
+	                 "recursive/1/b,"
 	                 "recursive/4,"
 	                 "recursive/4/c");
 	g_free (content);
@@ -1463,77 +1477,6 @@ test_monitored_atomic_replace (TrackerMinerFSTestFixture *fixture,
 	g_free (content);
 }
 
-static void
-test_changes_after_no_mtime_check (TrackerMinerFSTestFixture *fixture,
-                                   gconstpointer              data)
-{
-	gchar *content;
-
-	CREATE_FOLDER (fixture, "recursive");
-	CREATE_FOLDER (fixture, "recursive/1");
-	CREATE_FOLDER (fixture, "recursive/1/2");
-	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	CREATE_UPDATE_FILE (fixture, "recursive/1/2/b");
-
-	fixture_add_indexed_folder (fixture, "recursive",
-	                            TRACKER_DIRECTORY_FLAG_PRESERVE |
-	                            TRACKER_DIRECTORY_FLAG_CHECK_MTIME |
-	                            TRACKER_DIRECTORY_FLAG_CHECK_DELETED |
-	                            TRACKER_DIRECTORY_FLAG_RECURSE);
-
-	tracker_miner_start (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
-
-	content = fixture_get_content (fixture);
-	g_assert_cmpstr (content, ==,
-	                 "recursive,"
-	                 "recursive/1,"
-	                 "recursive/1/2,"
-	                 "recursive/1/2/b,"
-	                 "recursive/a");
-	g_free (content);
-
-	fixture_remove_indexed_folder (fixture, "recursive");
-
-	CREATE_FOLDER (fixture, "recursive/1/3");
-	CREATE_UPDATE_FILE (fixture, "recursive/c");
-	DELETE_FOLDER (fixture, "recursive/1/2");
-
-	fixture_add_indexed_folder (fixture, "recursive",
-	                            TRACKER_DIRECTORY_FLAG_MONITOR |
-	                            TRACKER_DIRECTORY_FLAG_RECURSE);
-	fixture_iterate (fixture);
-
-	content = fixture_get_content (fixture);
-	g_assert_cmpstr (content, ==,
-	                 "recursive,"
-	                 "recursive/1,"
-	                 "recursive/1/2,"
-	                 "recursive/1/2/b,"
-	                 "recursive/a");
-	g_free (content);
-
-	CREATE_UPDATE_FILE (fixture, "recursive/1/3/c");
-	CREATE_UPDATE_FILE (fixture, "recursive/1/2");
-	CREATE_UPDATE_FILE (fixture, "recursive/c");
-
-	while (!fixture_query_exists (fixture, "recursive/c"))
-		fixture_iterate (fixture);
-
-	content = fixture_get_content (fixture);
-	g_assert_cmpstr (content, ==,
-	                 "recursive,"
-	                 "recursive/1,"
-	                 "recursive/1/2,"
-	                 "recursive/1/2/b," /* FIXME: this file should disappear */
-	                 "recursive/1/3,"
-	                 "recursive/1/3/c,"
-	                 "recursive/a,"
-	                 "recursive/c");
-	g_free (content);
-}
-
 static void
 test_event_queue_create_and_update (TrackerMinerFSTestFixture *fixture,
                                     gconstpointer              data)
@@ -1555,21 +1498,15 @@ test_event_queue_create_and_update (TrackerMinerFSTestFixture *fixture,
 	g_assert_cmpstr (content, ==, "recursive");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 	UPDATE_FILE_ATOMIC (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==, "recursive");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
 	g_assert_cmpint (((TestMiner *) fixture->miner)->n_process_file, ==, 1);
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1599,16 +1536,9 @@ test_event_queue_create_and_delete (TrackerMinerFSTestFixture *fixture,
 	g_assert_cmpstr (content, ==, "recursive");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
-	g_assert_true (tracker_miner_fs_has_items_to_process (fixture->miner) == TRUE);
-
 	DELETE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
 	fixture_iterate (fixture);
 	g_assert_true (tracker_miner_fs_has_items_to_process (fixture->miner) == FALSE);
 }
@@ -1634,20 +1564,14 @@ test_event_queue_create_and_move (TrackerMinerFSTestFixture *fixture,
 	g_assert_cmpstr (content, ==, "recursive");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==, "recursive");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1680,14 +1604,11 @@ test_event_queue_update_and_update (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
 	test_miner_reset_counters ((TestMiner *) fixture->miner);
 
 	UPDATE_FILE_ATOMIC (fixture, "recursive/a", "b");
-	fixture_iterate_filter (fixture, 1);
 	CREATE_UPDATE_FILE (fixture, "b");
 	MOVE_FILE (fixture, "b", "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1695,9 +1616,7 @@ test_event_queue_update_and_update (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 	/* Coalescing desirable, but not mandatory */
 	g_assert_cmpint (((TestMiner *) fixture->miner)->n_process_file, >=, 1);
 
@@ -1732,12 +1651,8 @@ test_event_queue_update_and_delete (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	UPDATE_FILE_ATOMIC (fixture, "recursive/a", "b");
-	fixture_iterate_filter (fixture, 1);
 	DELETE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1745,9 +1660,7 @@ test_event_queue_update_and_delete (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==, "recursive");
@@ -1778,13 +1691,9 @@ test_event_queue_update_and_move (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	DELETE_FILE (fixture, "recursive/a");
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1792,9 +1701,7 @@ test_event_queue_update_and_move (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1827,12 +1734,8 @@ test_event_queue_delete_and_create (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	DELETE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1840,9 +1743,7 @@ test_event_queue_delete_and_create (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1875,14 +1776,10 @@ test_event_queue_move_and_update (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	test_miner_reset_counters ((TestMiner *) fixture->miner);
 
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 	UPDATE_FILE_ATOMIC (fixture, "recursive/b", "c");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1890,10 +1787,8 @@ test_event_queue_move_and_update (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
-	g_assert_cmpint (((TestMiner *) fixture->miner)->n_process_file, ==, 1);
+	fixture_iterate_timed (fixture, 1);
+	g_assert_cmpint (((TestMiner *) fixture->miner)->n_process_file, ==, 2);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1926,12 +1821,8 @@ test_event_queue_move_and_create_origin (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 	CREATE_UPDATE_FILE (fixture, "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1939,9 +1830,7 @@ test_event_queue_move_and_create_origin (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1975,12 +1864,8 @@ test_event_queue_move_and_delete (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 	DELETE_FILE (fixture, "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -1988,9 +1873,7 @@ test_event_queue_move_and_delete (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -2022,12 +1905,8 @@ test_event_queue_move_and_move (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 	MOVE_FILE (fixture, "recursive/b", "recursive/c");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -2035,9 +1914,7 @@ test_event_queue_move_and_move (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -2070,12 +1947,8 @@ test_event_queue_move_and_move_back (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_pause (TRACKER_MINER (fixture->miner));
-
 	MOVE_FILE (fixture, "recursive/a", "recursive/b");
-	fixture_iterate_filter (fixture, 1);
 	MOVE_FILE (fixture, "recursive/b", "recursive/a");
-	fixture_iterate_filter (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -2083,9 +1956,7 @@ test_event_queue_move_and_move_back (TrackerMinerFSTestFixture *fixture,
 	                 "recursive/a");
 	g_free (content);
 
-	tracker_miner_resume (TRACKER_MINER (fixture->miner));
-
-	fixture_iterate (fixture);
+	fixture_iterate_timed (fixture, 1);
 
 	content = fixture_get_content (fixture);
 	g_assert_cmpstr (content, ==,
@@ -2156,9 +2027,6 @@ main (gint    argc,
 
 	g_test_message ("Testing filesystem miner");
 
-	g_test_message ("SKIPPED");
-	return 77;
-
 	ADD_TEST ("/indexing-tree/recursive-indexing",
 	          test_recursive_indexing);
 	ADD_TEST ("/indexing-tree/non-recursive-indexing",
@@ -2214,8 +2082,6 @@ main (gint    argc,
 	          test_monitored_move);
 	ADD_TEST ("/monitored/atomic-replace",
 	          test_monitored_atomic_replace);
-	ADD_TEST ("monitored/changes-after-no-mtime-check",
-	          test_changes_after_no_mtime_check);
 
 	/* Tests for event queues */
 	ADD_TEST ("event-queue/create-and-update",
